// File: Settings.pas
//
// Description: TPrnSettings class containing virtual printer preferences

unit Settings;

interface

uses Windows, SysUtils;

const
  // The registry key (HKLM) where settings are stored
  RegistryKey = 'Your Virtual Printer Example';

type
  // Print job information transfer modes
  TTransferModes = (
    tmThroughCommandLine, // Through the command line
    tmThroughCopydata,    // Through the WM_COPYDATA message
    tmThroughClipboard    // Thtough the clipboard
  );

  // TPrnSettings class
  TPrnSettings = class
  public
    // Transfer mode related settings
    ClientAppPath: string;        // Full path to the client application
    ClientWndClass: string;       // Class name of the client application window
    ClientWndTitle: string;       // Title of the client application window
    ClipboardFormat: string;      // Name of the clipboard format
    MessageId: Integer;           // Numeric value for message identification
    TransferMode: TTransferModes; // Print job information transfer mode
    OutputPath: string;           // Output directory for genereated files

    // Property pages visibility settings
    HideEmfPage: BOOL;            // true if "Emf" page is hidden
    HideBmpPage: BOOL;            // true if "Bmp" page is hidden
    HidePngPage: BOOL;            // true if "Png" page is hidden
    HideJpegPage: BOOL;           // true if "Jpeg" page is hidden
    HideTiffPage: BOOL;           // true if "Tiff" page is hidden
    HidePdfPage: BOOL;            // true if "Pdf" page is hidden
    HideTxtPage: BOOL;            // true if "Txt" page is hidden
    HideRedirectPage: BOOL;       // true if "Redirect" page is hidden
    HideLanguagePage: BOOL;       // true if "Language" page is hidden

    // Constructor, sets the default values (settings)
    constructor Create;

    // Load settings from the registry
    function Load: Boolean;

    // Save settings to the registry
    function Save: Boolean;

  private
    // Set all parameters to defaults (without saving them)
    procedure LoadDefaults;
  end;

implementation

// Constructor, sets the default values (settings)
constructor TPrnSettings.Create;
begin
  LoadDefaults;
end;

// Set all parameters to defaults (without saving them)
procedure TPrnSettings.LoadDefaults;
begin
  ClientAppPath := 'X:\\YourFolder\\YourClient.exe';
  ClientWndClass := '';
  ClientWndTitle := 'Sample Client Application';
  ClipboardFormat := 'Your Virtual Printer Example clipboard format';
  MessageId := $8000;
  TransferMode := tmThroughCommandLine;
  OutputPath := '';

  HideEmfPage := false; HideBmpPage := false; HidePngPage := false; HideJpegPage := false;
  HideTiffPage := false; HidePdfPage := false; HideTxtPage := false;
  HideRedirectPage := false; HideLanguagePage := false;
end;

procedure ReadRegStr(Key: HKEY; const Name: string;
  var Parameter: string; const Len: Integer);
var
  Size: DWORD;
  Buf: PChar;
begin
  GetMem(Buf, Len); Size := Len;
  if RegQueryValueEx(Key, PChar(Name), nil, nil, PByte(Buf),
    @Size) = ERROR_SUCCESS then Parameter := Buf;
  FreeMem(Buf);
end;

procedure ReadRegDword(Key: HKEY; const Name: string; Parameter: PDWORD);
var
  Size: DWORD;
begin
  Size := SizeOf(DWORD);
  RegQueryValueEx(Key, PChar(Name), nil, nil, PByte(Parameter), @Size);
end;

// Load settings from the registry
function TPrnSettings.Load: Boolean;
var
  Key: HKEY;
begin
  Result := false;

  // Open the registry key
  if RegOpenKeyEx(HKEY_LOCAL_MACHINE, PChar('Software\' + RegistryKey),
    0, KEY_READ, Key) = ERROR_SUCCESS then
  begin
    // Reset all settings to defaults
    LoadDefaults;

    // Read transfer mode related settings
    ReadRegStr(Key, 'Client Application', ClientAppPath, MAX_PATH);
    ReadRegStr(Key, 'Target Window Class', ClientWndClass, 256);
    ReadRegStr(Key, 'Target Window Title', ClientWndTitle, 256);
    ReadRegStr(Key, 'Clipboard Format', ClipboardFormat, 256);
    ReadRegStr(Key, 'Output Directory', OutputPath, MAX_PATH);
    ReadRegDword(Key, 'Message ID', PDWORD(@MessageId));
    ReadRegDword(Key, 'Transfer Mode', PDWORD(@TransferMode));

    // Load property pages visibility settings
    ReadRegDword(Key, 'EMF: Hide page', PDWORD(@HideEmfPage));
    ReadRegDword(Key, 'BMP: Hide page', PDWORD(@HideBmpPage));
    ReadRegDword(Key, 'PNG: Hide page', PDWORD(@HidePngPage));
    ReadRegDword(Key, 'JPEG: Hide page', PDWORD(@HideJpegPage));
    ReadRegDword(Key, 'TIFF: Hide page', PDWORD(@HideTiffPage));
    ReadRegDword(Key, 'PDF: Hide page', PDWORD(@HidePdfPage));
    ReadRegDword(Key, 'TXT: Hide page', PDWORD(@HideTxtPage));
    ReadRegDword(Key, 'REDIRECT: Hide page', PDWORD(@HideRedirectPage));
    ReadRegDword(Key, 'LANGUAGE: Hide page', PDWORD(@HideLanguagePage));

    RegCloseKey(Key);
    Result := true;
  end;
end;

function WriteRegStr(Key: HKEY; const Name, Parameter: string): Integer;
begin
  Result := RegSetValueEx(Key, PChar(Name), 0, REG_SZ,
    PChar(Parameter), Length(Parameter) + 1);
end;

function WriteRegDword(Key: HKEY; const Name: string; const Parameter: DWORD): Integer;
begin
  Result := RegSetValueEx(Key, PChar(Name), 0, REG_DWORD,
    @Parameter, SizeOf(DWORD));
end;

// Save settings to the registry
function TPrnSettings.Save: Boolean;
var
  Key: HKEY;
  Res: Integer;
begin
  // Open the registry key
  Res := RegOpenKeyEx(HKEY_LOCAL_MACHINE, PChar('Software\' + RegistryKey),
    0, KEY_WRITE, Key);

  if Res = ERROR_SUCCESS then
  begin
    // Write transfer mode related settings
    Res := Res or WriteRegStr(Key, 'Client Application', ClientAppPath);
    Res := Res or WriteRegStr(Key, 'Target Window Class', ClientWndClass);
    Res := Res or WriteRegStr(Key, 'Target Window Title', ClientWndTitle);
    Res := Res or WriteRegStr(Key, 'Clipboard Format', ClipboardFormat);
    Res := Res or WriteRegStr(Key, 'Output Directory', OutputPath);
    Res := Res or WriteRegDword(Key, 'Message ID', DWORD(MessageID));
    Res := Res or WriteRegDword(Key, 'Transfer Mode', DWORD(TransferMode));

    // Load property pages visibility settings
    Res := Res or WriteRegDword(Key, 'EMF: Hide page', DWORD(HideEmfPage));
    Res := Res or WriteRegDword(Key, 'BMP: Hide page', DWORD(HideBmpPage));
    Res := Res or WriteRegDword(Key, 'PNG: Hide page', DWORD(HidePngPage));
    Res := Res or WriteRegDword(Key, 'JPEG: Hide page', DWORD(HideJpegPage));
    Res := Res or WriteRegDword(Key, 'TIFF: Hide page', DWORD(HideTiffPage));
    Res := Res or WriteRegDword(Key, 'PDF: Hide page', DWORD(HidePdfPage));
    Res := Res or WriteRegDword(Key, 'TXT: Hide page', DWORD(HideTxtPage));
    Res := Res or WriteRegDword(Key, 'REDIRECT: Hide page', DWORD(HideRedirectPage));
    Res := Res or WriteRegDword(Key, 'LANGUAGE: Hide page', DWORD(HideLanguagePage));

    RegCloseKey(Key);
  end;

  Result := Res = ERROR_SUCCESS;
end;

end.
